class PcCaptureSession
{
  static initialize(fOnLoad=null)
  {
    PcCaptureSession.shot = [];

    PcCaptureSession.currentShot = 0;
    PcCaptureSession.numShots = 0;

    PcCaptureSession.idShot = null;
    PcCaptureSession.idFrame = null;

    PcCaptureSession.defaultShot = [];
    PcCaptureSession.defaultCurrentShot = 0;
    PcCaptureSession.defaultNumShots = 0;

    PcCaptureSession.resources = [];

    PcCaptureSession.intializeResources(()=>{
      PcCaptureSession.loadDefaults();
      if (fOnLoad) fOnLoad();
    });

    PcCaptureSession.framing = new PcFraming();
    PcCaptureSession.framing.initialize();
  }

  static intializeResources(fOnLoad=null)
  {
    let urlsToLoad = [];
    let k=0;

    let testLoading = (url)=>{
        for (let i=0; i<urlsToLoad.length; i++) {if (urlsToLoad[i].url==url) urlsToLoad[i].loaded = true;}
        for (let i=0; i<urlsToLoad.length; i++) {if (urlsToLoad[i].loaded==false) return;}
        if (fOnLoad) fOnLoad();
    }

    urlsToLoad.push({name:"filter_general_vintagecover", url:PcConfig.host + "ressources/filter_general_vintagecover.png", loaded:false});
    urlsToLoad.push({name:"filter_general_whiteglow", url:PcConfig.host + "ressources/filter_general_whiteglow.png", loaded:false});
    urlsToLoad.push({name:"filter_general_yellowglow", url:PcConfig.host + "ressources/filter_general_yellowglow.png", loaded:false});

    if (PcConfig.scenario.mode=="photobooth")
    {
      urlsToLoad.push({name:"defaults_f1", url:PcConfig.host + "ressources/booth_frame1.jpg?x="+Math.random(), loaded:false}); // la meuf par défaut 1 2 3 4
      urlsToLoad.push({name:"defaults_f2", url:PcConfig.host + "ressources/booth_frame2.jpg?x="+Math.random(), loaded:false});
      urlsToLoad.push({name:"defaults_f3", url:PcConfig.host + "ressources/booth_frame3.jpg?x="+Math.random(), loaded:false});
      urlsToLoad.push({name:"defaults_f4", url:PcConfig.host + "ressources/booth_frame4.jpg?x="+Math.random(), loaded:false});
    }
    else {
      urlsToLoad.push({name:"defaults_f1", url:PcConfig.host + "ressources/kiosk_frame1.jpg?x="+Math.random(), loaded:false});
      urlsToLoad.push({name:"defaults_f2", url:PcConfig.host + "ressources/kiosk_frame2.jpg?x="+Math.random(), loaded:false});
      urlsToLoad.push({name:"defaults_f3", url:PcConfig.host + "ressources/kiosk_frame3.jpg?x="+Math.random(), loaded:false});
      urlsToLoad.push({name:"defaults_f4", url:PcConfig.host + "ressources/kiosk_frame4.jpg?x="+Math.random(), loaded:false});
    }

    for (let i=0; i<urlsToLoad.length; i++) {PcCaptureSession.loadResource(urlsToLoad[i].name, urlsToLoad[i].url, testLoading);}

    if (urlsToLoad.length==0) {if (fOnLoad) fOnLoad();}
  }

  static loadDefaults()
  {
    this.addDefaultFrame(PcCaptureSession.getResource("defaults_f1"), true);
    this.nextDefaultShot();

    this.addDefaultFrame(PcCaptureSession.getResource("defaults_f2"), true);
    this.nextDefaultShot();

    this.addDefaultFrame(PcCaptureSession.getResource("defaults_f3"), true);
    this.nextDefaultShot();

    this.addDefaultFrame(PcCaptureSession.getResource("defaults_f4"), true);

    setTimeout(()=>this.defaultFilter(PcConfig.screen.format.filter), 2000);
  }

  static nextShot()
  {
    if (PcCaptureSession.shot[PcCaptureSession.currentShot])
    {
        PcCaptureSession.currentShot++;
    }
  }

  static nextDefaultShot()
  {
    if (PcCaptureSession.defaultShot[PcCaptureSession.defaultCurrentShot])
    {
        PcCaptureSession.defaultCurrentShot++;
    }
  }

  static addFrame(data, dataIsCanvas=false)
  {
    if (!PcCaptureSession.shot[PcCaptureSession.currentShot])
    {
      PcCaptureSession.shot[PcCaptureSession.currentShot] = new PcCaptureSessionShot();
      PcCaptureSession.numShots++;
    }

    PcCaptureSession.shot[PcCaptureSession.currentShot].addFrame(data, dataIsCanvas);

    if (PcCaptureSession.currentShot == 0)
    {
      if (PcCaptureSession.shot[0].frame.length == 1) // addFrame: if shot has only one frame...
      {
        this.idFrame = new PcCaptureSessionFrame(data, dataIsCanvas); // addFrame: ... then make idFrame this very frame
      }
    }

  }

  static addDefaultFrame(data, dataIsCanvas=false)
  {
    if (!PcCaptureSession.defaultShot[PcCaptureSession.defaultCurrentShot])
    {
      PcCaptureSession.defaultShot[PcCaptureSession.defaultCurrentShot] = new PcCaptureSessionShot();
      PcCaptureSession.defaultNumShots++;
    }

    PcCaptureSession.defaultShot[PcCaptureSession.defaultCurrentShot].addFrame(data, dataIsCanvas);
  }

  /* applyFraming now takes predefined shots_landmarks
   if applicable they will be reused during the final call of PcFraming.apply()
   to avoid recomputing landmarks detection
  */
  static async applyFraming(onApplyFraming, predef_shots_landmarks, faceRatio, centerRatio, frameSize, error) // applies to PcCaptureSession.shot[] array
  {
    var shots_landmarks = [];
    let cur_predef_shot_landmarks;
   
    PcConfig.dbg(FRAMINGDBG, "PCS: applyFraming: predef_shots_landmarks", JSON.stringify(predef_shots_landmarks));
    PcConfig.dbg(FRAMINGDBG, "PCS: applyFraming: faceRatio",  faceRatio, "centerRatio", JSON.stringify(centerRatio), "frameSize", JSON.stringify(frameSize), "au mieux",  !alert);
  
    // call the PcCaptureSessionShot framing for each shot
    for (let i=0; i<PcCaptureSession.shot.length; i++)  {
      // ensure cur_predef_shot_landmarks sanity
      if (predef_shots_landmarks == null || predef_shots_landmarks.length < i+1) { /* fixme: hack for predef_shots_landmarks sizing vs unpredictible frame.length! */
        cur_predef_shot_landmarks = null;
      } else {
        cur_predef_shot_landmarks = predef_shots_landmarks[i];
      }
      PcConfig.dbg(FRAMINGDBG, "PCS: applyFraming: -> cur_predef_shot_landmarks", i, "=", JSON.stringify(cur_predef_shot_landmarks));
      //accumulate the resulting shot landmark
      shots_landmarks.push(
        await PcCaptureSession.shot[i].framing(
          PcCaptureSession.framing /* it's a class instance ! bad inheritance! yuk!*/,
          cur_predef_shot_landmarks,
          faceRatio,
          centerRatio,
          frameSize,
          error)
      );
      PcConfig.dbg(FRAMINGDBG, "PCS: applyFraming -> shots_landmarks are now", JSON.stringify(shots_landmarks));
    }
    //call applyFraming callback aka onApplyFraming with the resulting shots_lanmarks -> this will speed up any call to makeID()
    onApplyFraming(shots_landmarks);

    return shots_landmarks;
  }

  static async createIdShot(onApplyFraming, predef_frame_landmarks, faceRatio, centerRatio, frameSize) // applies to PcCaptureSession.idFrame !
  {
    //PcCaptureSession.idShot = new PcCaptureSessionShot();
    //PcCaptureSession.idShot.addFrame(PcCaptureSession.idFrame.canvas, true);
    PcConfig.dbg(FRAMINGDBG, "PCS: createIdShot: predef_frame_landmarks", JSON.stringify(predef_frame_landmarks));
    PcConfig.dbg(FRAMINGDBG, "PCS: createIdShot: faceRatio",  faceRatio, "centerRatio", centerRatio, "frameSize", JSON.stringify(frameSize));
    
    await PcCaptureSession.idFrame.framing(PcCaptureSession.framing, predef_frame_landmarks, faceRatio, centerRatio, frameSize, true);

    onApplyFraming();
  }

  static filter(filter)
  {
    PcCaptureSession.shot.forEach((shot, index) => shot.filter(filter, index));
  }

  static defaultFilter(filter)
  {
    PcCaptureSession.defaultShot.forEach((shot, index) => shot.filter(filter, index));
  }

  static reset()
  {
    PcCaptureSession.shot.forEach(shot => shot.delete());
    PcCaptureSession.shot.splice(0);
    PcCaptureSession.currentShot = 0;
    PcCaptureSession.numShots = 0;
  }

  static resetDefault()
  {
    PcCaptureSession.defaultShot.forEach(shot => shot.delete());
    PcCaptureSession.defaultShot.splice(0);
    PcCaptureSession.defaultCurrentShot = 0;
    PcCaptureSession.defaultNumShots = 0;
  }

  static loadResource(name, imgUrl, fOnLoad=null)
  {
    if (imgUrl=="") return;

    PcCaptureSession.resources[name] = new Image();

    if (fOnLoad)
    {
      PcCaptureSession.resources[name].onload = ()=>{fOnLoad(imgUrl)};
      PcCaptureSession.resources[name].onerror = ()=>{fOnLoad(imgUrl)};
    }

    PcCaptureSession.resources[name].src = imgUrl;
  }

  static getResource(name)
  {
    let img = PcCaptureSession.resources[name];
    let canvas = document.createElement('canvas');
    canvas.width = img.naturalWidth;
    canvas.height = img.naturalHeight;
    canvas.getContext('2d').drawImage(img, 0, 0);
    return canvas;
  }

  static cloneCanvas(oldCanvas)
  {
    var newCanvas = document.createElement('canvas');
    var context = newCanvas.getContext('2d');

    newCanvas.width = oldCanvas.width;
    newCanvas.height = oldCanvas.height;

    context.drawImage(oldCanvas, 0, 0);

    return newCanvas;
  }

}
