class PcCaptureSessionFrame
{
  constructor(data, dataIsCanvas=false, onLoad)
  {
    this.canvas = document.createElement('canvas');
    this.context = this.canvas.getContext('2d');
    this.filtered = {};
    this.maxSourceWidth = 10000;

    if (dataIsCanvas)
    {
      this.canvas.width = data.width;
      this.canvas.height = data.height;
      this.context.drawImage(data, 0, 0, data.width, data.height);

      if (PcConfig.scenario.cameraOrientation=="portrait" || PcConfig.scenario.cameraOrientation=="portraitInverted")
      {
        let destW = this.canvas.width;
        let destH = this.canvas.height;

        let newcanvas = this.cloneCanvas(this.canvas);

        this.context.clearRect(0,0,this.canvas.width,this.canvas.height);
        this.canvas.width = destH;
        this.canvas.height = destW;
        this.context.save();
        this.context.translate(this.canvas.width/2, this.canvas.height/2);

        if (PcConfig.scenario.cameraOrientation=="portrait")
        {
          this.context.rotate(-90*Math.PI/180);
        }
        else {
          this.context.rotate(90*Math.PI/180);
        }

        this.context.drawImage(newcanvas, -destW/2, -destH/2, destW, destH);
        this.context.restore();
      }

      if (this.onLoad) this.onLoad();
    }
    else {
      this.imageURL = data;
      this.onLoad = onLoad;
      this.image = document.createElement('img');
      this.image.onload = () => this.imageLoaded();
      this.image.src = data;
    }
  }

  imageLoaded()
  {
    var w = this.image.naturalWidth;
    var h = this.image.naturalHeight;
    var destW, destH;

    if (w>this.maxSourceWidth)
    {
      destW = this.maxSourceWidth;
      destH = destW * h/w;
    }
    else {
      destW = w;
      destH = h;
    }

    this.canvas.width = destW;
    this.canvas.height = destH;
    this.context.drawImage(this.image, 0, 0, destW, destH);

    if (PcConfig.scenario.cameraOrientation=="portrait" || PcConfig.scenario.cameraOrientation=="portraitInverted")
    {
      let newcanvas = this.cloneCanvas(this.canvas);

      this.context.clearRect(0,0,this.canvas.width,this.canvas.height);
      this.canvas.width = destH;
      this.canvas.height = destW;
      this.context.save();
      this.context.translate(this.canvas.width/2, this.canvas.height/2);

      if (PcConfig.scenario.cameraOrientation=="portrait")
      {
        this.context.rotate(-90*Math.PI/180);
      }
      else {
        this.context.rotate(90*Math.PI/180);
      }

      this.context.drawImage(newcanvas, -destW/2, -destH/2, destW, destH);
      this.context.restore();
    }

    delete this.image;

    if (this.onLoad) this.onLoad();
  }

  filter(filter, index)
  {
    if (!this.filtered.hasOwnProperty(filter.id))
    {
      this.filtered[filter.id] = this.applyFilter(filter, index);
    }
  }
  /* per Frame framing now takes predef_frame_landmarks.
  it is used to potentially avoid recomputing landmarks 
  */

  async framing(framing, predef_frame_landmarks, faceRatio, centerRatio, frameSize, alert)
  {
    PcConfig.dbg(FRAMINGDBG, "PCSF: framing: predef_frame_landmarks", JSON.stringify(predef_frame_landmarks));
    PcConfig.dbg(FRAMINGDBG, "PCSF: framing: faceRatio",  faceRatio, "centerRatio", JSON.stringify(centerRatio), "frameSize", JSON.stringify(frameSize), "au mieux",  !alert);
    let frame_landmarks;
    [ this.canvas, frame_landmarks ]   = await framing.apply(
      predef_frame_landmarks, 
      this.canvas,
      faceRatio,
      centerRatio,
      frameSize,
      alert); // framing is a class instance from PcCaptureSession, yuk

      PcConfig.dbg(FRAMINGDBG, "PCSF: framing -> frame_landmarks =", JSON.stringify(frame_landmarks));

      // return the Frame landmarks
      return frame_landmarks;
  }

  cloneCanvas(oldCanvas)
  {
    var newCanvas = document.createElement('canvas');
    var context = newCanvas.getContext('2d');

    newCanvas.width = oldCanvas.width;
    newCanvas.height = oldCanvas.height;

    context.drawImage(oldCanvas, 0, 0);

    return newCanvas;
  }

  applyFilter(filter, index)
  {
    var filter = new PcFilter(filter);
    return filter.apply(this.canvas, index);
  }

  delete()
  {
    this.context = null;
    this.canvas = null;
    this.image = null;

    //delete this.image;

    for (var p in this.filtered) if (this.filtered.hasOwnProperty(p))
    {
      this.filtered[p] = null;
    }

    this.filtered = null;
  }
}
