class PcCaptureSessionShot
{
  constructor()
  {
    this.frame = [];
  }

  addFrame(data, dataIsCanvas=false)
  {
    this.frame.push(new PcCaptureSessionFrame(data, dataIsCanvas, this.onLoadFrame));
  }

  onLoadFrame()
  {
  }

  filter(filter, index)
  {
    this.frame.forEach(frame => frame.filter(filter, index));
  }

  /* per Shot framing now takes predef_frames_landmarks.
  they are used to potentially avoid recomputing landmarks for specific frames (from one shot - which can be animated with multiple frames)
  */

  async framing(framing, predef_frames_landmarks, faceRatio, centerRatio, frameSize, alert) /* framing is a PcFraming instance from PcCaptureSession, yuk */
  {
    var frames_landmarks = [];

    for (let i=0; i<this.frame.length; i++) // for all frames in shot (animated )
    {
      // ensure cur_predef_frame_landmarks sanity
      let cur_predef_frame_landmarks;
      if (predef_frames_landmarks == null || predef_frames_landmarks.length < i+1) { /* fixme: hack for predef_frames_landmarks sizing vs unpredictible frame.length! */
        cur_predef_frame_landmarks = null;
      } else {
        cur_predef_frame_landmarks = predef_frames_landmarks[i];
      }
      /* call the framing per frame 
      accumulate the frame landmarks
      */
      frames_landmarks.push(
        await this.frame[i].framing(framing, cur_predef_frame_landmarks , faceRatio, centerRatio, frameSize, alert)
      );
      PcConfig.dbg(FRAMINGDBG, "PCSS: framing -> frames_landmarks are now", JSON.stringify(frames_landmarks));

    }

    //return the frames_landmarks for this shot
    return frames_landmarks;
  }

  delete()
  {
    this.frame.forEach(frame => frame.delete());
    this.frame.splice(0);
    this.frame = null;
  }
}
