class PcFilter
{
    constructor(filter)
    {
      this.filterName = filter.filterName;
      this.settings = filter.settings;
      this.canvas = document.createElement('canvas');
      this.context = this.canvas.getContext('2d');
    }

    apply(canvas, variation)
    {
      var w = canvas.width;
      var h = canvas.height;
      this.canvas.width = w;
      this.canvas.height = h;
      this.context.drawImage(canvas, 0, 0);

      switch(this.filterName)
      {
          case "none" : case "natural" : return this.filterNatural(); break;

          case "blackandwhite" : return this.filterBlackandwhite(); break;
          case "desaturation" : return this.filterDesaturate(); break;
          case "polaroid" : return this.filterPolaroid(); break;
          case "popart" : return this.filterPopart2(variation); break;
          case "captur" : return this.filterCaptur(variation); break;
          case "vintage" : return this.filterVintage(); break;

          case "insta_brannan" : return this.filterInstaBrannan(); break;
          case "insta_earlybird" : return this.filterInstaEarlybird(); break;
          case "insta_inkwell" : return this.filterInstaInkwell(); break;
          case "insta_lofi" : return this.filterInstaLofi(); break;
          case "insta_xproii" : return this.filterInstaXproii(); break;

          case "pixels_solangegrey" : return this.filterPixelsSolangegrey(); break;
          case "pixels_diagonallines" : return this.filterPixelsDiagonallines(); break;
          case "pixels_greendiagonallines" : return this.filterPixelsGreendiagonallines(); break;
          case "pixels_extraoffsetblue" : return this.filterPixelsExtraoffsetblue(); break;

          //case "framing" : return this.filterFraming(); break;
          default : return this.canvas;
      }

      return canvas;
    }

    filterNatural()
    {
      return this.canvas;
    }

    filterBlackandwhite()
    {
      var canvas = this.cloneCanvas(this.canvas);

      canvas.getContext('2d').globalCompositeOperation = 'overlay';
      canvas.getContext('2d').globalAlpha = 0.7;
      canvas.getContext('2d').drawImage(canvas, 0, 0);

      this.grayScale(canvas);
      return canvas;
    }

    filterPolaroid()
    {
      var canvas = this.cloneCanvas(this.canvas);
      var white = PcCaptureSession.getResource("filter_general_whiteglow");
      var yellow = PcCaptureSession.getResource("filter_general_yellowglow");
      var cover = PcCaptureSession.getResource("filter_general_vintagecover");

      var c1 = this.cloneCanvas(this.canvas);
      c1.getContext('2d').globalCompositeOperation = 'overlay';
      c1.getContext('2d').globalAlpha = 0.9;
      c1.getContext('2d').drawImage(white, 0, 0, c1.width, c1.height);
      c1.getContext('2d').globalCompositeOperation = 'overlay';
      c1.getContext('2d').globalAlpha = 0.4;
      c1.getContext('2d').drawImage(yellow, 0, 0, c1.width, c1.height);

      var c2 = this.cloneCanvas(c1);
      this.grayScale(c2);

      canvas.getContext('2d').globalCompositeOperation = 'overlay';
      canvas.getContext('2d').globalAlpha = 1;
      canvas.getContext('2d').drawImage(c2, 0, 0, canvas.width, canvas.height);
      canvas.getContext('2d').globalCompositeOperation = 'multiply';
      canvas.getContext('2d').globalAlpha = 0.9;
      canvas.getContext('2d').drawImage(c1, 0, 0, canvas.width, canvas.height);

      canvas.getContext('2d').globalCompositeOperation = 'source-over';
      canvas.getContext('2d').globalAlpha = 1;
      canvas.getContext('2d').drawImage(cover, 0, 0, canvas.width, canvas.height);

      return canvas;
    }

    filterVintage()
    {
      var canvas = this.cloneCanvas(this.canvas);
      var white = PcCaptureSession.getResource("filter_general_whiteglow");
      var yellow = PcCaptureSession.getResource("filter_general_yellowglow");
      var cover = PcCaptureSession.getResource("filter_general_vintagecover");

      var c1 = this.cloneCanvas(this.canvas);

      c1.getContext('2d').globalCompositeOperation = 'overlay';
      c1.getContext('2d').globalAlpha = 0.7;
      c1.getContext('2d').drawImage(white, 0, 0, c1.width, c1.height);
      c1.getContext('2d').globalCompositeOperation = 'multiply';
      c1.getContext('2d').globalAlpha = 0.9;
      c1.getContext('2d').drawImage(yellow, 0, 0, c1.width, c1.height);
      c1.getContext('2d').globalCompositeOperation = 'source-over';
      c1.getContext('2d').globalAlpha = 0.6;
      c1.getContext('2d').drawImage(cover, 0, 0, c1.width, c1.height);

      c1.getContext('2d').globalCompositeOperation = 'overlay';
      c1.getContext('2d').globalAlpha = 0.3;
      c1.getContext('2d').drawImage(canvas, 0, 0, c1.width, c1.height);

      this.saturation(c1, -50);
      this.contrast(c1, 50);

      return c1;
    }

    filterDesaturate()
    {
      var canvas = this.cloneCanvas(this.canvas);
      var c1 = this.cloneCanvas(this.canvas);

      this.grayScale(c1);

      canvas.getContext('2d').globalCompositeOperation = 'overlay';
      canvas.getContext('2d').globalAlpha = 1;
      canvas.getContext('2d').drawImage(c1, 0, 0);

      return canvas;
    }

    filterPopart(variation)
    {
      let colorsHex = this.settings.split("|");
      let color = this.hexToRgb(colorsHex[variation])

      var canvas = this.cloneCanvas(this.canvas);
      this.leveling(canvas, color);
      return canvas;
    }

    filterPopart2(variation)
    {
      let settings;

      if (this.settings.replace(/\s/g, "")!="")
      {
        settings = this.settings.split("|");
      }
      else {
        settings = ["f15a25", "eda505", "0089a9", "00a99e", "000000", "155"];
      }

      let bgcolor = this.hexToRgb(settings[variation]);
      let color = this.hexToRgb(settings[4]);
      let average = parseInt(settings[5], 10);

      if (average==0) average = 155;

      let canvas = this.cloneCanvas(this.canvas);
      let context = canvas.getContext('2d');
      let canvasWidth  = canvas.width;
      let canvasHeight = canvas.height;
      let imageData = context.getImageData(0, 0, canvasWidth, canvasHeight);
      let data = imageData.data;

      data = this.bd_grayscale(data, canvasWidth, canvasHeight);
      data = this.bd_contrastAndBrightness(data, canvasWidth, canvasHeight, 1, 0.85, true);
      data = this.bd_equalize(data, canvasWidth, canvasHeight);
      data = this.bd_levelize(data, canvasWidth, canvasHeight, average, color, bgcolor);

      canvas.getContext('2d').putImageData(imageData, 0, 0);

      return canvas;
    }

    filterCaptur(variation)
    {
      let settings;

      if (this.settings.replace(/\s/g, "")!="")
      {
        settings = this.settings.split("|");
      }
      else {
        settings = ["fffff", "155"];
      }

      let color = this.hexToRgb(settings[0]);
      let average = parseInt(settings[1]);

      if (average==0) average = 155;

      let canvas = this.cloneCanvas(this.canvas);
      let context = canvas.getContext('2d');
      let canvasWidth  = canvas.width;
      let canvasHeight = canvas.height;
      let imageData = context.getImageData(0, 0, canvasWidth, canvasHeight);
      let data = imageData.data;

      data = this.bd_grayscale(data, canvasWidth, canvasHeight);
      data = this.bd_contrastAndBrightness(data, canvasWidth, canvasHeight, 1, 0.85, true);
      data = this.bd_equalize(data, canvasWidth, canvasHeight);
      data = this.bd_levelize_trans(data, canvasWidth, canvasHeight, average, color);

      canvas.getContext('2d').putImageData(imageData, 0, 0);

      return canvas;
    }

    filterInstaBrannan()
    {
      var canvas = this.cloneCanvas(this.canvas);

      canvas.getContext('2d').filter = "sepia(.5) contrast(1.4)";
      canvas.getContext('2d').drawImage(canvas, 0, 0);

      canvas.getContext('2d').globalCompositeOperation = 'lighten';
      canvas.getContext('2d').fillStyle="rgba(161, 44, 199, .31)";
      canvas.getContext('2d').fillRect( 0, 0, canvas.width, canvas.height);

      return canvas;
    }

    filterInstaEarlybird()
    {
      var canvas = this.cloneCanvas(this.canvas);

      canvas.getContext('2d').filter = "sepia(.2) contrast(.9)";
      canvas.getContext('2d').drawImage(canvas, 0, 0);

      var gradient = canvas.getContext('2d').createRadialGradient(canvas.width/2, canvas.height/2, 0, canvas.width/2, canvas.height/2, Math.max(canvas.width, canvas.height)/2*1.1);
      gradient.addColorStop(0.2, "#d0ba8e");
      gradient.addColorStop(0.85, "#360309");
      gradient.addColorStop(1, "#1d0210");

      canvas.getContext('2d').globalCompositeOperation = 'overlay';
      canvas.getContext('2d').fillStyle = gradient;
      canvas.getContext('2d').fillRect(0, 0, canvas.width, canvas.height);

      return canvas;
    }

    filterInstaInkwell()
    {
      var canvas = this.cloneCanvas(this.canvas);

      canvas.getContext('2d').filter = "sepia(.3) contrast(1.1) brightness(1.1) grayscale(1)";
      canvas.getContext('2d').drawImage(canvas, 0, 0);

      return canvas;
    }

    filterInstaLofi()
    {
      var canvas = this.cloneCanvas(this.canvas);

      canvas.getContext('2d').filter = "saturate(1.1) contrast(1.5)";
      canvas.getContext('2d').drawImage(canvas, 0, 0);

      var gradient = canvas.getContext('2d').createRadialGradient(canvas.width/2, canvas.height/2, 0, canvas.width/2, canvas.height/2, Math.max(canvas.width, canvas.height)/2*1.5);
      gradient.addColorStop(0.6, "transparent");
      gradient.addColorStop(1, "#222222");

      canvas.getContext('2d').globalCompositeOperation = 'multiply';
      canvas.getContext('2d').fillStyle = gradient;
      canvas.getContext('2d').fillRect(0, 0, canvas.width, canvas.height);

      return canvas;
    }

    filterInstaXproii()
    {
      var canvas = this.cloneCanvas(this.canvas);

      canvas.getContext('2d').filter = "sepia(.3)";
      canvas.getContext('2d').drawImage(canvas, 0, 0);

      var gradient = canvas.getContext('2d').createRadialGradient(canvas.width/2, canvas.height/2, 0, canvas.width/2, canvas.height/2, Math.max(canvas.width, canvas.height)/2*1.2);
      gradient.addColorStop(0.4, "#e6e7e0");
      gradient.addColorStop(1, "rgba(43, 42, 161, .5)");

      canvas.getContext('2d').globalCompositeOperation = 'color-burn';
      canvas.getContext('2d').fillStyle = gradient;
      canvas.getContext('2d').fillRect(0, 0, canvas.width, canvas.height);

      return canvas;
    }

    filterPixelsSolangegrey()
    {
      var canvas = this.cloneCanvas(this.canvas);

      let imgData = canvas.getContext('2d').getImageData(0, 0, canvas.width, canvas.height);
      let newImgData = pixelsJS.filterImgData(imgData, "solange_grey");
      canvas.getContext('2d').putImageData(newImgData, 0, 0);

      return canvas;
    }

    filterPixelsExtraoffsetblue()
    {
      var canvas = this.cloneCanvas(this.canvas);

      let imgData = canvas.getContext('2d').getImageData(0, 0, canvas.width, canvas.height);
      let newImgData = pixelsJS.filterImgData(imgData, "extra_offset_blue");
      canvas.getContext('2d').putImageData(newImgData, 0, 0);

      return canvas;
    }

    filterPixelsDiagonallines()
    {
      var canvas = this.cloneCanvas(this.canvas);

      let imgData = canvas.getContext('2d').getImageData(0, 0, canvas.width, canvas.height);
      let newImgData = pixelsJS.filterImgData(imgData, "diagonal_lines");
      canvas.getContext('2d').putImageData(newImgData, 0, 0);

      return canvas;
    }

    filterPixelsGreendiagonallines()
    {
      var canvas = this.cloneCanvas(this.canvas);

      let imgData = canvas.getContext('2d').getImageData(0, 0, canvas.width, canvas.height);
      let newImgData = pixelsJS.filterImgData(imgData, "green_diagonal_lines");
      canvas.getContext('2d').putImageData(newImgData, 0, 0);

      return canvas;
    }

    ////////////////////////////////////////////////////////////////////////////
    // SUB FUNCTIONS
    ////////////////////////////////////////////////////////////////////////////

    grayScale(canvas)
    {
      var context = canvas.getContext('2d');
      var imgData = context.getImageData(0, 0, canvas.width, canvas.height);
      var pixels  = imgData.data;
      for (var i = 0, n = pixels.length; i < n; i += 4)
      {
        var grayscale = pixels[i] * .3 + pixels[i+1] * .59 + pixels[i+2] * .11;
        pixels[i  ] = grayscale;
        pixels[i+1] = grayscale;
        pixels[i+2] = grayscale;
      }

      context.putImageData(imgData, 0, 0);
    }

    leveling(canvas, color)
    {
      var context = canvas.getContext('2d');
      var imgData = context.getImageData(0, 0, canvas.width, canvas.height);
      var pixels  = imgData.data;

      var total = 0;
      var average = 0;
      var a1, a2;

      for (var i = 0, n = pixels.length; i < n; i += 4)
      {
        var grayscale = pixels[i] * .33 + pixels[i+1] * .33 + pixels[i+2] * .33;
        total += grayscale;
      }

      average = total/(pixels.length/4);

      for (var i = 0, n = pixels.length; i < n; i += 4)
      {
        var grayscale = pixels[i] * .33 + pixels[i+1] * .33 + pixels[i+2] * .33;
        var c;

        if (grayscale>average)
        {
          c = color;
        }
        else {
          c = [0, 0, 0];
        }

        pixels[i  ] = c[0];
        pixels[i+1] = c[1];
        pixels[i+2] = c[2];
      }

      context.putImageData(imgData, 0, 0);
    }

    contrast(canvas, contrast)
    {
      var context = canvas.getContext('2d');
      var imgData = context.getImageData(0, 0, canvas.width, canvas.height);
      var data = imgData.data;
      var factor = (259 * (contrast + 255)) / (255 * (259 - contrast));

      for(var i=0;i<data.length;i+=4)
      {
          data[i] = factor * (data[i] - 128) + 128;
          data[i+1] = factor * (data[i+1] - 128) + 128;
          data[i+2] = factor * (data[i+2] - 128) + 128;
      }

      context.putImageData(imgData, 0, 0);
    }

    saturation (canvas, saturation)
    {
      var context = canvas.getContext('2d');
      var imgData = context.getImageData(0, 0, canvas.width, canvas.height);
      var d = imgData.data;
      var value = saturation/100;

      for (var i = 0; i < d.length; i += 4)
      {
        var r = d[i];
        var g = d[i + 1];
        var b = d[i + 2];
        var gray = 0.2989*r + 0.5870*g + 0.1140*b; //weights from CCIR 601 spec
        d[i] = -gray * value + d[i] * (1+value);
        d[i+1] = -gray * value + d[i+1] * (1+value);
        d[i+2] = -gray * value + d[i+2] * (1+value);
        //normalize over- and under-saturated values
        if(d[i] > 255) d[i] = 255;
        if(d[i+1] > 255) d[i] = 255;
        if(d[i+2] > 255) d[i] = 255;
        if(d[i] < 0) d[i] = 0;
        if(d[i+1] < 0) d[i] = 0;
        if(d[i+2] < 0) d[i] = 0;
    }

    context.putImageData(imgData, 0, 0);
  };

  bd_getGrey(r,g,b)
  {
    return  Math.round(0.2126*r + 0.7152*g + 0.0722*b);
  };

  bd_grayscale(data, width, height)
  {
      for (var y = 0; y < height; ++y)
      {
        for (var x = 0; x < width; ++x)
        {
          let i = (y * width + x) * 4;
          let r = data[i];
          let g = data[i+1];
          let b = data[i+2];
          let a = data[i+3];
          let grey = this.bd_getGrey(r,g,b);

          data[i] = data[i+1] = data[i+2] = grey;
        }
      }
      return data;
  };

  bd_contrastAndBrightness(data, width, height, contrast, brightness, onBlack=false)
  {
    contrast = contrast*128;
    brightness = brightness*255;

    let factor = (259*(contrast+255))/(255*(259-contrast))

    for (var y = 0; y < height; ++y)
    {
      for (var x = 0; x < width; ++x)
      {
        let i = (y * width + x) * 4;
        let r = data[i];
        let g = data[i+1];
        let b = data[i+2];
        let a = data[i+3];

        let nr = Math.round(factor*(r-128) + 128 + brightness);
        let ng = Math.round(factor*(g-128) + 128 + brightness);
        let nb = Math.round(factor*(b-128) + 128 + brightness);

        if (onBlack)
        {
          let grey = this.bd_getGrey(r,g,b);
          let ratio = (255-grey)/255;
          ratio = Math.pow(ratio,2);
          nr = r*(1-ratio) + nr*ratio;
          ng = g*(1-ratio) + ng*ratio;
          nb = b*(1-ratio) + nb*ratio;
        }

        nr > 255 ? nr = 255 : (nr<0 ? nr=0 : 0);
        ng > 255 ? ng = 255 : (ng<0 ? ng=0 : 0);
        nb > 255 ? nb = 255 : (nb<0 ? nb=0 : 0);

        data[i] = nr;
        data[i+1] = ng;
        data[i+2] = nb;
      }
    }

    return data;
  }

  bd_equalize(data, width, height)
  {
    //min and max vals
    let min = 255;
    let max = 0;

    for (let y = 0; y < height; ++y)
    {
      for (let x = 0; x < width; ++x)
      {
        let i = (y * width + x) * 4;
        let r = data[i];
        let g = data[i+1];
        let b = data[i+2];
        let a = data[i+3];
        let grey = this.bd_getGrey(r, g, b);
        if (grey<min) min =grey;
        if (grey>max) max =grey;
      }
    }

    //equalizing
    for (let y = 0; y < height; ++y)
    {
      for (let x = 0; x < width; ++x)
      {
        let i = (y * width + x) * 4;
        let r = data[i];
        let g = data[i+1];
        let b = data[i+2];
        let a = data[i+3];
        let grey = this.bd_getGrey(r, g, b);
        let newGrey = Math.round( (grey-min) * 255 / (max-min) );
        data[i] = data[i+1] = data[i+2] = newGrey;
      }
    }

    return data;
  }

  bd_levelize(data, width, height, average, color=[255,255,255], bgcolor=[0,0,0])
  {
    for (var y = 0; y < height; ++y)
    {
      for (var x = 0; x < width; ++x)
      {
        let i = (y * width + x) * 4;
        let r = data[i];
        let g = data[i+1];
        let b = data[i+2];
        let a = data[i+3];
        let grey = this.bd_getGrey(r,g,b)

        if (a>0)
        {
          if (grey>average)
          {
            data[i] = bgcolor[0];
            data[i+1] = bgcolor[1];
            data[i+2] = bgcolor[2];
            data[i+3] = 255;
          }
          else {
            data[i] = color[0];
            data[i+1] = color[1];
            data[i+2] = color[2];
            data[i+3] = 255;
          }
        }

      }
    }
    return data;
  }

  bd_levelize_trans(data, width, height, average, color=[255,255,255])
  {
    for (var y = 0; y < height; ++y)
    {
      for (var x = 0; x < width; ++x)
      {
        let i = (y * width + x) * 4;
        let r = data[i];
        let g = data[i+1];
        let b = data[i+2];
        let a = data[i+3];
        let grey = this.bd_getGrey(r,g,b)

        if (a>0)
        {
          if (grey>average)
          {
            data[i] = color[0];
            data[i+1] = color[1];
            data[i+2] = color[2];
            data[i+3] = 255;
          }
          else {
            data[i] = 0;
            data[i+1] = 0;
            data[i+2] = 0;
            data[i+3] = 0;
          }
        }

      }
    }
    return data;
  }

  hexToRgb(hex) {
    hex = hex.replace("#", '');
    var bigint = parseInt(hex, 16);
    var r = (bigint >> 16) & 255;
    var g = (bigint >> 8) & 255;
    var b = bigint & 255;

    return [r, g, b];
  }

  cloneCanvas(oldCanvas)
  {
    var newCanvas = document.createElement('canvas');
    var context = newCanvas.getContext('2d');

    newCanvas.width = oldCanvas.width;
    newCanvas.height = oldCanvas.height;

    context.drawImage(oldCanvas, 0, 0);

    return newCanvas;
  }
}
