class PcLabel extends React.Component
{
  constructor(props)
  {
    super(props);
    this.labelObj = React.createRef();
    this.labelContainer = React.createRef();
    this.state = {label:props.label};
  }

  getStringWidth(text, font, size)
  {
    var div = document.createElement('span');
    div.style.position = "absolute";
    div.style.display = "inline";
    div.style.fontFamily = font;
    div.style.fontSize = size;
    div.style.color = "#ffffff";
    div.textContent = text;

    document.body.appendChild(div);
    var width = div.getBoundingClientRect().width;
    document.body.removeChild(div);

    return width;
  }

  getWidth()
  {
    return this.labelContainer.current.offsetWidth + parseFloat(this.labelContainer.current.style.marginLeft) + parseFloat(this.labelContainer.current.style.marginRight);
  }

  changeHeight(height)
  {
    this.labelContainer.current.style.height = height;
  }

  show()
  {
    this.labelContainer.current.style.display="table";
  }

  hide()
  {
    this.labelContainer.current.style.display="none";
  }

  updateLabel(label, fade=false)
  {
    this.setState({label:label});

    if (fade)
    {
      this.labelObj.current.style.transition = "";
      this.labelObj.current.style.opacity=0;
      setTimeout(()=>{this.labelObj.current.style.transition="all 0.5s ease-out"; this.labelObj.current.style.opacity=1;}, 100);
    }
  }

  labelToLink(label)
  {
    label = label+"";
    if (!label) return "";
    label = label.replace("<doc>", "<a href='#' style='color: inherit;' id='doc'>");
    label = label.replace("</doc>", "</a>");
    return label;
  }

  onClickLabel(e)
  {
    if (e.target.id=="doc")
    {
      if (this.props.onClickDoc) this.props.onClickDoc();
    }
  }

  render()
  {
    var width, height;
    var valign, halign;
    var x, y;
    var position;
    var shadow;

    this.props.width ? (width = this.props.width) : (width="auto");
    this.props.height ? (height = this.props.height) : (height="auto");
    this.props.valign ? (valign = this.props.valign) : (valign="middle");
    this.props.halign ? (halign = this.props.halign) : (halign="center");
    this.props.x ? (x = this.props.x) : (x=0);
    this.props.y ? (y = this.props.y) : (y=0);
    this.props.position ? (position = this.props.position) : (position="absolute");

    if (this.props.shadow)
    {
      shadow = this.props.shadow
    } else {
      if (this.props.title)
      {
        if (PcConfig.scenario.titlesStyle=="shadow")
        {
          let v = 2 * PcConfig.renderSize;
          shadow = v + "px " + v + "px " + (2*v) + "px " + "rgba(0, 0, 0, 0.75)";
        } else {
          shadow = "";
        }
      } else {
        shadow = "";
      }
    }

    return e('div',
      {
        className:"pclabelcontainer",
        ref:this.labelContainer,
        onClick:()=>{if (this.props.onClick) this.props.onClick()},
        style:
        {
          display:"table",
          position:position,
          opacity:this.props.noAnimation?1:0,
          animation:this.props.noAnimation?"":PcConfig.scenario.elementsAnimationStyle,
          animationFillMode:this.props.noAnimation?"":"forwards",
          animationDelay:this.props.noAnimation?"":this.props.showDelay,
          left:this.props.x,
          top:this.props.y,
          wordWrap: "break-word",
          overflow:"hidden",
          width:this.props.width,
          height:this.props.height,
          marginLeft:this.props.spacingH,
          marginRight:this.props.spacingH,
          marginTop:this.props.spacingV,
          marginBottom:this.props.spacingV
        }
      },
      e('div',
        {
          className:"pclabel",
          ref:this.labelObj,
          onClick:(e)=>{this.onClickLabel(e)},
          onTouchEnd:(e)=>{this.onClickLabel(e)},
          style:
            {
              position:"relative",
              fontFamily:PcConfig.scenario.fontFamily,
              fontSize:this.props.fontSize,
              color:this.props.color,
              verticalAlign: valign,
              textAlign: halign,
              wordWrap: "break-word",
              display:"table-cell",
              width:this.props.width,
              whiteSpace: "pre-line",
              textShadow: shadow,
              fontWeight: this.props.bold ? "bold" : "normal",
              transition: "all 0.5s ease-out",
            },
          dangerouslySetInnerHTML:{__html:this.labelToLink(this.state.label)}
        }
      )
    );
  }

}
